#!/bin/sh

# confidea wcap3 upgrade script

usage()
{
	cat << EOF
    usage: $0 options

    This script updates the root file system.

    OPTIONS:
     [ -k      path to kernel file   ]
     [ -i      path to rootfs file   ]
     [ -p      path to progress file ]
     [ -s      path to status file   ]
EOF
}

print_error()
{
	printf "Update: ERROR    step %s/%s - %s\n" "$UPGRADE_STEP" "$STEPS_TOTAL" "$1" >> $PROGRESS_FILE
	printf "{\"done\":true,\"error\":\"true\"}" > $STATUS_FILE
}

print_progress()
{
	printf "Update: PROGRESS step %s/%s - %s\n" "$UPGRADE_STEP" "$STEPS_TOTAL" "$1" >> $PROGRESS_FILE
	printf "{\"done\":false,\"progress\":\"%s\"}" "$1" > $STATUS_FILE
}

check_file()
{
	if [ -z "$2" ] || [ ! -f $2 ]
	then
		print_error "$1 ($2) can't be found"
		exit 1
	fi
}

KERNEL_FILE="uImage"
ROOTFS_FILE="rootfs.ubi"

PROGRESS_FILE=$(tty)    # path to progress file
STATUS_FILE="/dev/null" # path to status file

STEPS_TOTAL="5"
UPGRADE_STEP="0"

O=`getopt -q -- hk:i:p:s: "$@"`
eval set -- "$O"
while true; do
	case $1 in
		-h)
			usage
			exit 0
			shift;;
		-k)
			KERNEL_FILE=$2
			shift 2;;
		-i)
			ROOTFS_FILE=$2
			shift 2;;
		-p)
			PROGRESS_FILE=$2
			if [ -f $PROGRESS_FILE ] ; then
				>| $PROGRESS_FILE
			fi
			shift 2;;
		-s)
			STATUS_FILE=$2
			if [ -f $STATUS_FILE ] ; then
				>| $STATUS_FILE
			fi
			shift 2;;
		--)
			shift; break;;
		*)
			shift; break;;
	esac
done

# Pre-install checks
if [ -z /etc/board ] || \
   [ ! -f /etc/board ]
then
	print_error "This upgrade can only be installed on a 2.x Confidea G3 unit."
	exit 1
fi
BOARD_TYPE=$(cat /etc/board)

if [ "$BOARD_TYPE" != "confidea_g3_upgrade" ] && \
   [ "$BOARD_TYPE" != "confidea_g3_golden" ] && \
   [ "$BOARD_TYPE" != "confidea_g3_production" ]
then
	print_error "This upgrade can only be installed on a Confidea G3 unit."
	exit 1
fi

if [ "$BOARD_TYPE" == "confidea_g3_golden" ]
then
	dd if=/dev/zero of=/sys/devices/platform/ocp/48030000.spi/spi_master/spi1/spi1.1/eeprom
fi

# Note: an update will change the application - number
# The number is stored to a temporary file (removed on reboot)
# to make sure we don't accidently update the wrong location
# in case a second update is done, without rebooting.
if [ ! -e /tmp/application ]
then
	fw_printenv -n application > /tmp/application
fi
APPLICATION=$(cat /tmp/application)

# currently first partition?
if [ $APPLICATION -ne 1 ]; then
	KERNEL="Kernel_A"
	KERNELSIZE="KernelSize_A"
	ROOTFS="Rootfs_A"
	APPLICATION=1
else
	KERNEL="Kernel_B"
	KERNELSIZE="KernelSize_B"
	ROOTFS="Rootfs_B"
	APPLICATION=2
fi;

KERNEL_PARTITION="/dev/$(cat /proc/mtd | grep $KERNEL | cut -f 0 -d ':')"
ROOTFS_PARTITION="/dev/$(cat /proc/mtd | grep $ROOTFS | cut -f 0 -d ':')"

if [ -z "$KERNEL_PARTITION" ] || [ -z "$ROOTFS_PARTITION" ]
then
	print_error "missing valid mtd device."
	exit 1
fi

# check if required files exist
check_file "kernel" $KERNEL_FILE
check_file "rootfs" $ROOTFS_FILE

UPGRADE_STEP="1"
print_progress "Installing kernel $KERNEL_FILE on $KERNEL_PARTITION"
flash_erase -q $KERNEL_PARTITION 0 0
nandwrite -q -p $KERNEL_PARTITION $KERNEL_FILE

UPGRADE_STEP="2"
print_progress "Installing rootfs $ROOTFS_FILE on $ROOTFS_PARTITION"
ubiformat $ROOTFS_PARTITION -y -q -f rootfs.ubi

UPGRADE_STEP="3"
# calculate uImage size in hex
print_progress "Update uboot variables: kernelsize $KERNELSIZE"
KSIZE=`du $KERNEL_FILE | cut -f1`
KSIZE=`awk 'BEGIN { rounded = "'"$KSIZE"'" * 1024; printf "%x", rounded }'`
fw_setenv $KERNELSIZE $KSIZE

UPGRADE_STEP="4"
print_progress "Update uboot variables: application $APPLICATION"
fw_setenv application $APPLICATION

UPGRADE_STEP="5"
print_progress "Update finished"

# signal confidea update success
exit 0
